html, body{
    margin: 1em;
    /* background-color: #1a1a1a;
    color: white; */
    font-family: monospace;
    /* font-size: larger; */
}
form{
    margin: 1em;
    padding: 1em;
}
.title{
    font-size: larger;
    margin-left: 40px;
    margin-bottom: 0.25em;
}
.field label{
    display: inline-block;
    width: 40px;
    margin-bottom: 6px;
}
.result{
    margin-top: 6px;
}
.column {
    float: left;
    width: 50%;
}
/* Clear floats after the columns */
.row:after {
    content: "";
    display: table;
    clear: both;
}
